#ifndef INC_LISTA_SECUENCIAL
#define INC_LISTA_SECUENCIAL
#include "listarecursiva.h"
#include <cstdlib>
#include <exception>
   using namespace std;
template <class T>
class ListaSecuencial {
   public:
   // Constructoras
      /**
       * Representa la lista secuencial vaca
       */
      ListaSecuencial();
   
      /**
       * Inserta un elemento en la posicin actual
       */
      void insertar(const T &e);
   
      /**
       * Elimina el elemento que ocupa la posicin actual. El siguiente
       * elemento pasa a ocupar dicha posicin, si existe, o la posicin
       * actual queda ms all del final, si se extrae el ltimo.
       * Cuando, antes de extraer, la posicin actual se encuentra ms all 
       * del final de la lista, se lanza la excepcin ErrorFinLista
       */
      void extraer();
   
      /**
       * Modifica la posicin actual haciendo que seale al siguiente
       * elemento en la lista, si existe, o la posicin
       * actual queda ms all del final, si se extrae el ltimo.
       * Cuando, antes de avanzar, la posicin actual se encuentra ms all 
       * del final de la lista, se lanza la excepcin ErrorFinLista
       */
      void avanzar();
   
      /**
       * Modifica la posicin actual haciendo que seale al elemento
       * que le precede en la lista, si existe.
       * Cuando la posicin actual se encuentra al principio de la lista
       * se lanza la excepcin ErrorInicioLista
       */
      void retroceder();
   
      /**
       * Modifica la posicin actual haciendo que seale al principio
       * de la lista, dando acceso al primer elemento si la lista no
       * est vaca
       */
      void inicio();
   
      /**
       * Modifica la posicin actual haciendo que seale al final
       * de la lista, detrs del ltimo elemento
       */
      void fin();
   
   // Observadoras
      /**
       * Devuelve verdadero si la posicin actual se encuentra
       * al principio de la lista secuencial
       */
      bool esInicio() const;
   
      /**
       * Devuelve verdadero si la posicin actual se encuentra
       * ms all del final de la lista secuencial
       */
      bool esFin() const;
   
      /**
       * Devuelve el elemento actual. Si la posicin actual se encuentra
       * ms all del final de la lista, lanza la excepcin ErrorFinLista
       */
      T examinar() const;
   
      /**
       * Devuelve verdadero si el nmero de elementos es distinto de cero
       */
      bool hayElementos() const;
   
      /**
       * Devuelve el nmero de elementos de la lista secuencial
       */
      int cardinal() const;
      class ErrorInicioLista: public exception {};
      class ErrorFinLista: public exception {};
      
   private:
      // Sulista de los elementos anteriores al actual
      ListaRecursiva<T> anteriores;
      // Sublista del elemento actual y los posteriores
      ListaRecursiva<T> posteriores;
      int numElementos;
};

/**
 * Representa la lista secuencial vaca
 */
template <class T>
ListaSecuencial<T>::ListaSecuencial() {
   anteriores  = ListaRecursiva<T>();
   posteriores = ListaRecursiva<T>();
   numElementos= 0;
};

/**
 * Crea una lista aadiendo al principio de una lista ya existente
 * un nuevo elemento con semntica de alias
 */
template <class T>
void ListaSecuencial<T>::insertar(const T &e) {
   posteriores = ListaRecursiva<T>(e,posteriores);
   numElementos++;
};

/**
 * Elimina el elemento que ocupa la posicin actual. El siguiente
 * elemento pasa a ocupar dicha posicin, si existe, o la posicin
 * actual queda ms all del final, si se extrae el ltimo.
 * Cuando, antes de extraer, la posicin actual se encuentra ms all 
 * del final de la lista, se lanza la excepcin ErrorFinLista
 */
template <class T>
void ListaSecuencial<T>::extraer() { 
   if (esFin()) throw ErrorFinLista();
   
   posteriores = posteriores.resto();
   numElementos--;
}; 

/**
 * Modifica la posicin actual haciendo que seale al siguiente
 * elemento en la lista, si existe, o la posicin
 * actual queda ms all del final, si se extrae el ltimo.
 * Cuando, antes de avanzar, la posicin actual se encuentra ms all 
 * del final de la lista, se lanza la excepcin ErrorFinLista
 */
template <class T>
void ListaSecuencial<T>::avanzar() {
   if (esFin()) throw ErrorFinLista();
   
   anteriores = ListaRecursiva<T>(posteriores.primero(),anteriores);
   posteriores  = posteriores.resto();
};

/**
 * Modifica la posicin actual haciendo que seale al elemento
 * que le precede en la lista, si existe.
 * Cuando la posicin actual se encuentra al principio de la lista
 * se lanza la excepcin ErrorInicioLista
 */
template <class T>
void ListaSecuencial<T>::retroceder() {
   if (esInicio()) throw ErrorInicioLista();
   
   posteriores = ListaRecursiva<T>(anteriores.primero(),posteriores);
   anteriores  = anteriores.resto();
};

/**
 * Modifica la posicin actual haciendo que seale al principio
 * de la lista, dando acceso al primer elemento si la lista no
 * est vaca
 */
template <class T>
void ListaSecuencial<T>::inicio() {
   while (!esInicio()) retroceder();
};

/**
 * Modifica la posicin actual haciendo que seale al final
 * de la lista, detrs del ltimo elemento
 */
template <class T>
void ListaSecuencial<T>::fin() {
   while (!esFin()) avanzar();
};

/**
 * Devuelve verdadero si la posicin actual se encuentra
 * al principio de la lista secuencial
 */
template <class T>
bool ListaSecuencial<T>::esInicio() const {
   return !anteriores.hayElementos();
};

/**
 * Devuelve verdadero si la posicin actual se encuentra
 * ms all del final de la lista secuencial
 */
template <class T>
bool ListaSecuencial<T>::esFin() const {
   return !posteriores.hayElementos();
};

/**
 * devuelve el elemento actual. Si la posicin actual se encuentra
 * ms all del final de la lista, lanza la excepcin ErrorFinLista
 */
template <class T>
T ListaSecuencial<T>::examinar() const {
   if (esFin()) throw ErrorFinLista();
  
   return posteriores.primero();
};

/**
 * Devuelve verdadero si el nmero de elementos es distinto de cero
 */
template <class T>
bool ListaSecuencial<T>::hayElementos() const {
   return numElementos != 0;
};

/**
 * Devuelve el nmero de elementos de la lista secuencial
 */
template <class T>
int ListaSecuencial<T>::cardinal() const {
   return numElementos;
};

#endif
